#pragma once

const long MAX_LVITEMLEN = 128;

// Class:   CTVItemData 
// Purpose: Simple class that represents data contained in the node of a tree control.
//          An instance of this class is created for every tree control node
// 
// Methods:
//      GetTreeNodeItemInfo(...) -- Called by CVirtualTreeDlg::OnTvnGetdispinfoList
//      SetSelectedState(...) -- Called by CLVSelStateDlg::OnNMClickList
class CTVItemData : public CLockableCS 
{
public:

    enum { TVID_COLUMN_ONE = 0, TVID_COLUMN_TWO = 1, TVID_COLUMN_THREE = 2 };

    CTVItemData( LPCTSTR szName, const LONG lID, const LONG lParentID )
      : m_sName( szName )
         , m_lID( lID )
         , m_lParentID( lParentID )
         , m_bExpand( FALSE )
    {
       TRACE( _T("Name: %s\tParentID: %d\tID: %d\n"), szName, lParentID, lID );
    }

    CTVItemData::CTVItemData(const CTVItemData& rhs)
      : m_lID( rhs.m_lID )
         , m_lParentID( rhs.m_lParentID )
         , m_sName( rhs.m_sName )
         , m_bExpand( rhs.m_bExpand )
    {
    }

    ~CTVItemData()
    {
    }

// Operations
public:

   LPCTSTR GetName( )
   {
      return m_sName;
   }

   const LONG GetID( )
   {
      return m_lID;
   }

   const BOOL IsExpand( )
   {
      return m_bExpand;
   }
 
    // Method:  GetTreeViewItemInfo
    // Purpose: Called by CVirtualTreeDlg::OnTvnGetdispinfoList
    //          Used to supply the display data for the virtual
    //          tree control
    //
    void GetTreeViewItemInfo( LPTVITEM pItem )
    {
        // Thread safe locking
        CAutoLockT<CTVItemData> lock(this);
        pItem->pszText = m_sName.GetBuffer( );
        //StringCchCopy( pItem->pszText, MAX_LVITEMLEN, m_sName );
    }

// Attributes
private:
   LONG     m_lID;
   LONG     m_lParentID;
   CString  m_sName;
   BOOL     m_bExpand;
};

// Turn a std::vector into a lockable vector
// NOTE: For this example, a vector as a data structure makes sense
// because we initially populate the tree data one time.
// In the real world if the tree contents changes, you may wish
// to use something like a list when items are frequently updated.
class CTVItemDataList 
    : public std::vector< CTVItemData* >
    , public CLockableCS 
{
public:
	CTVItemDataList() {}
	~CTVItemDataList() {}
};


class CTVItemDataMMap 
    : public std::multimap< LONG, CTVItemData >
    , public CLockableCS 
{
public:
	CTVItemDataMMap() {}

   CTVItemDataMMap( CTVItemDataMMap& rhs )
   {
      insert( rhs.begin( ), rhs.end( ) );
   }

	~CTVItemDataMMap() {}
};

typedef std::multimap< LONG, CTVItemData >&	TVITEMDATA_MMAP_REF;

