// CTVItemDataMgr.cpp : Ctor/Dtor and Accessor implementations for CTVItemDataMgr class
//

#include "stdafx.h"
#include "TVItemData.h"
#include "TVItemDataMgr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CTVItemDataMgr::CTVItemDataMgr()
   : m_hRetrieveThread( NULL )
      , m_hShutdownEvent( ::CreateEvent( NULL, TRUE, FALSE, NULL ) )
      , m_lMaxCount( 1000000 )
      , m_lMultiplier( 5 )
      , m_lMaxDepth( 20 )
      , m_lCurrentDepth( 0 )
      , m_lRetrievalState( TVIDM_NOTRETRIEVED )
      , m_lID( 0 )
{
}

CTVItemDataMgr::~CTVItemDataMgr()
{
   ShutdownRetrieveThread();

   ::CloseHandle( m_hShutdownEvent );
   m_hShutdownEvent = NULL;
}

// Method:  SetDrive
// Purpose: Sets the Drive letter 
void CTVItemDataMgr::SetDriveLetter( LPCTSTR szDriveLetter )
{
   CAutoLockT< CLockableCS > lock( &m_csDriveLetter );
   m_sDriveLetter = szDriveLetter;
}

LPCTSTR CTVItemDataMgr::GetDriveLetter( )
{
   CAutoLockT< CLockableCS > lock( &m_csDriveLetter );
   return m_sDriveLetter;
}


// Method:  SetMaxCount
// Purpose: Sets the MaxCount (thread safe)
void CTVItemDataMgr::SetMaxCount( const LONG lMaxCount )
{
   ::InterlockedExchange( &m_lMaxCount, lMaxCount );
}

// Method:  SetMultiplier
// Purpose: Sets the Multiplier (thread safe)
void CTVItemDataMgr::SetMultiplier( const LONG lMultiplier )
{
   ::InterlockedExchange( &m_lMultiplier, lMultiplier );
}

// Method:  SetDepth
// Purpose: Sets the Depth (thread safe)
void CTVItemDataMgr::SetMaxDepth( const LONG lMaxDepth )
{
   ::InterlockedExchange( &m_lMaxDepth, lMaxDepth );
}

const LONG CTVItemDataMgr::GetCurrentDepth( )
{
   return ::InterlockedExchange( &m_lCurrentDepth, m_lCurrentDepth );
}
const LONG CTVItemDataMgr::IncCurrentDepth( )
{
   return ::InterlockedIncrement( &m_lCurrentDepth );
}

const LONG CTVItemDataMgr::DecCurrentDepth( )
{
   return ::InterlockedDecrement( &m_lCurrentDepth );
}

// Method:  SetRetrievalState
// Purpose: Sets the current retrieval state (thread safe)
void CTVItemDataMgr::SetRetrievalState( const LONG lState )
{
   ::InterlockedExchange( &m_lRetrievalState, lState );
}

// Method:  GetNextID
// Purpose: Returns the Next ID (thread safe)
const LONG CTVItemDataMgr::GetNextID( )
{
   return ::InterlockedIncrement( &m_lID );
}

// Method:  ResetID
// Purpose: Sets the ID to zero (thread safe)
void CTVItemDataMgr::ResetID( )
{
   InterlockedExchange( &m_lID, 0 );
}

// Method:  GetMaxCount
// Purpose: Returns the MaxCount (thread safe)
const LONG CTVItemDataMgr::GetMaxCount()
{
   return ::InterlockedExchange( &m_lMaxCount, m_lMaxCount );
}

// Method:  GetMultiplier
// Purpose: Returns the Multiplier (thread safe)
const LONG CTVItemDataMgr::GetMultiplier()
{
   return ::InterlockedExchange( &m_lMultiplier, m_lMultiplier );
}

// Method:  GetMaxDepth
// Purpose: Returns the depth (thread safe)
const LONG CTVItemDataMgr::GetMaxDepth()
{
   return ::InterlockedExchange( &m_lMaxDepth, m_lMaxDepth );
}

// Method:  GetRetrievalState
// Purpose: Returns the current retrieval state (thread safe)
const UINT CTVItemDataMgr::GetRetrievalState( )
{
   return ::InterlockedExchange( &m_lRetrievalState, m_lRetrievalState );
}

// Method:  GetShutdownEvent
// Purpose: Returns the shutdown event handle
HANDLE& CTVItemDataMgr::GetShutdownEvent()
{
   return m_hShutdownEvent;
}