// CTVItemDataMgrPublic.cpp : Public methods for CTVItemDataMgr class used
// by CVirtualTreeDlgeval
//

#include "stdafx.h"
#include "TVItemData.h"
#include "TVItemDataMgr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// Method:  ApplyMultiplier
// Purpose: Called by CVirtualTreeDlg::OnBnClickedRetrieved
//          after CTVItemDataMgr::Retrieve method.  Applies the 
//          specified multiplier to the retrieved directory items
void CTVItemDataMgr::ApplyMultiplier( )
{
   if( 1 < GetMultiplier( ) )
   {

      CAutoLockT< CTVItemDataMMap > lock( &m_TVItemDataMMap );
      CTVItemDataMMap temp( m_TVItemDataMMap );
      for( int nIndex = 1; // Start multiplying at two or more
         nIndex < GetMultiplier( );
         nIndex++ )
      {
            m_TVItemDataMMap.insert( temp.begin( ), temp.end( ) );
      }
   }
}


// Method:  Cancel
// Purpose: Called by CVirtualTreeDlg::OnBnClickedCancel
//          Shutsdown the secondary thread if running
//          and clears the TVItem data list
HRESULT CTVItemDataMgr::Cancel()
{
   ShutdownRetrieveThread();

   ClearTVItemDataMMap();

   SetRetrievalState( TVIDM_NOTRETRIEVED );

   return S_OK;
}

// Method:  Populate
// Purpose: Called within the Retrieve button to add items to
// the tree control.
HRESULT CTVItemDataMgr::Populate( CTreeCtrl& ctlTree, size_t& ruCount )
{
   HRESULT hr = S_OK;

   // Delete any tree items
   ctlTree.DeleteAllItems();

   // Lock the Tree control window
   ctlTree.LockWindowUpdate();

   // Lock the multimap
   CAutoLockT< CTVItemDataMMap > lock( &m_TVItemDataMMap );

   // Walk through the multi-map and add the virtual items
   // to the tree control
   BOOL bExpand = FALSE;
	InsertTreeNode( ctlTree, 1, TVI_ROOT, bExpand );

   // Add a wait cursor
   CWaitCursor wait;

   // Unlock the Tree control window
   ctlTree.UnlockWindowUpdate();

   // Retrieve the total count
   ruCount = m_TVItemDataMMap.size( );

   return hr;
}

// Method:  Retrieve
// Purpose: Called within CLVSelStateDlg::OnBnClickedRetrieve()
// Spawns a secondary thread which adds items to the LVItem data list
// and then waits for the thread to finish
HRESULT CTVItemDataMgr::Retrieve( LPCTSTR szDriveLetter, 
                                 const LONG lMaxCount,
                                 const LONG lMultiplier,
                                 const LONG lMaxDepth )
{
   long lCount = static_cast<LONG>( m_TVItemDataMMap.size( ) );

   HRESULT hr = S_OK;
   UINT uRetrieveTID = 0;

   // Stop and clean up if there is an existing thread
   ShutdownRetrieveThread();
   ClearTVItemDataMMap();
   ::ResetEvent( m_hShutdownEvent );

   SetDriveLetter( szDriveLetter );
   SetMaxCount( lMaxCount );
   SetMultiplier( lMultiplier );
   SetMaxDepth( lMaxDepth );

   ResetID( );

   SetRetrievalState( TVIDM_INPROGRESS );

   if( NULL == ( m_hRetrieveThread
                  = (HANDLE) _beginthreadex( NULL,
                                             0,
                                             RetrieveThread,
                                             this,
                                             0,
                                             &uRetrieveTID )))
   {
      return HRESULT_FROM_WIN32( ::GetLastError() );
   }

   // Process messages and wait for the thread to finish
   return MsgWaitForThread( m_hRetrieveThread );
}
