// CTVItemDataMgrThread.cpp : Thread implementation for CTVItemDataMgr class
//

#include "stdafx.h"
#include "TVItemData.h"
#include "TVItemDataMgr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// Method:  ClearTVItemDataMMap
// Purpose: Deletes any CTVItemData within the multimap
void CTVItemDataMgr::ClearTVItemDataMMap()
{
   CAutoLockT< CTVItemDataMMap > lock( &m_TVItemDataMMap );
   m_TVItemDataMMap.clear();
}

// Method:  MsgWaitForThread
// Purpose: Processes messages and waits for the specify thread
HRESULT CTVItemDataMgr::MsgWaitForThread( HANDLE hThread )
{
   HRESULT hr = S_OK;
   BOOL bContinue = TRUE;
   DWORD dwSignalled = 0;

   CWaitCursor wait;

	// Process main thread messages
	MSG msg = { 0 };

	while ( bContinue )
	{
		dwSignalled = MsgWaitForMultipleObjects( 1,
												   &hThread,
												   FALSE,
												   INFINITE,
												   QS_ALLINPUT);

		switch ( dwSignalled )
		{
		case WAIT_OBJECT_0:
         // Thread exited, so bail
		   bContinue = FALSE;
			break;
		case WAIT_OBJECT_0 + 1:
         // Process windows messages
         bContinue = ProcessMessages();
         wait.Restore(); // Restore the wait cursor
			break;
      default:
			return HRESULT_FROM_WIN32( ::GetLastError() );
		}
   }
   return hr;
}		

// Method:  ProcessMessages
// Purpose: Processes windows messages
BOOL CTVItemDataMgr::ProcessMessages( )
{
   MSG msg = { 0 };

   while( ::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) )
	{
		if( WM_QUIT == msg.message )
		{
			// Quit message received
			SetEvent( m_hShutdownEvent );
			return FALSE;
		}
		else
		{
			// Translate and dispatch the message
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
	}
   return TRUE;
}

// Method:  RetrieveThread
// Purpose: Secondary Thread which adds Max Count items to the
// TVItem Data list
UINT WINAPI CTVItemDataMgr::RetrieveThread( LPVOID lpContext )
{
   CTVItemDataMgr* pTVItemDataMgr = static_cast< CTVItemDataMgr* >( lpContext );

   pTVItemDataMgr->AddFoldersRecursively( pTVItemDataMgr->GetDriveLetter( ),
      pTVItemDataMgr->GetDriveLetter( ),
      pTVItemDataMgr->GetNextID( ) );

   pTVItemDataMgr->SetRetrievalState( TVIDM_RETRIEVED );

   return 0;
}

// Method:  ShutdownRetrieveThread
// Purpose: Closes the Sets the MaxCount (thread safe)
void CTVItemDataMgr::ShutdownRetrieveThread()
{
	// Close the Retrieve thread
	if( NULL != m_hRetrieveThread )
	{
		::SetEvent( m_hShutdownEvent );
		   
      if ( WAIT_TIMEOUT == WaitForSingleObject( m_hRetrieveThread, 1000 ) ) 
      {
         TRACE( _T("Using TerminateThread on Retrieve Thread\n") );
         TerminateThread( m_hRetrieveThread, -1000 );
      }

      ::CloseHandle( m_hRetrieveThread );
		m_hRetrieveThread = NULL;
   }
}
